"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.extractFollows = void 0;
const data_with_position_1 = require("data-with-position");
const kind_of_1 = __importDefault(require("kind-of"));
const lodash_1 = __importDefault(require("lodash"));
const extractFollows = (dataWithPosition, document) => {
    const followsWithPosition = dataWithPosition.follows;
    if (!lodash_1.default.isUndefined(followsWithPosition)) {
        const rawValue = (0, data_with_position_1.getValue)(followsWithPosition);
        const position = (0, data_with_position_1.getPosition)(followsWithPosition);
        if (lodash_1.default.isNull(rawValue)) {
            // ignore null value
        }
        else if (!lodash_1.default.isString(rawValue)) {
            document.message(`‘follows’ has to be a string, ${(0, kind_of_1.default)(rawValue)} given. Value ignored.`, position, "litvis:frontmatter-follows");
        }
        else if (rawValue.match(/\n/g)) {
            document.message(`‘follows’ cannot contain newlines. Value ignored.`, position, "litvis:frontmatter-follows");
        }
        else {
            const normalizedValue = rawValue.trim();
            if (normalizedValue !== rawValue) {
                document.info(`Surrounded spaces in ‘follows’ were trimmed.`, position, "litvis:frontmatter-follows");
            }
            return {
                value: normalizedValue,
                position,
            };
        }
    }
    return {};
};
exports.extractFollows = extractFollows;
//# sourceMappingURL=extractFollows.js.map