"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.lintElm = void 0;
const data_with_position_1 = require("data-with-position");
const kind_of_1 = __importDefault(require("kind-of"));
const lodash_1 = __importDefault(require("lodash"));
const supportedProperties = ["dependencies", "source-directories"];
const lintElm = (dataWithPosition, document) => {
    if (!lodash_1.default.isUndefined(dataWithPosition.elm)) {
        const elm = (0, data_with_position_1.getValue)(dataWithPosition.elm);
        if (lodash_1.default.isNull(elm)) {
            // ignore null value
        }
        else if (!lodash_1.default.isPlainObject(elm)) {
            document.message(`‘elm’ has to be an object, ${(0, kind_of_1.default)(elm)} given. Value ignored.`, (0, data_with_position_1.getPosition)(dataWithPosition.elm), "litvis:frontmatter:elm:dependencies");
        }
        else {
            const unusedKeys = lodash_1.default.without(lodash_1.default.keys(elm), ...supportedProperties);
            unusedKeys.forEach((k) => {
                document.message(`‘elm.${k}’ is not supported and so ignored. Supported properties: ${supportedProperties.join(", ")}.`, (0, data_with_position_1.getPosition)(dataWithPosition.elm[k]), "litvis:frontmatter:elm");
            });
        }
        if (dataWithPosition.dependencies && (!elm || !elm.dependencies)) {
            const elmPosition = (0, data_with_position_1.getPosition)(dataWithPosition.elm);
            const dependenciesPosition = (0, data_with_position_1.getPosition)(dataWithPosition.dependencies);
            if (dependenciesPosition.start.line > elmPosition.end.line) {
                document.message(`It seems that you are missing indentation before ‘dependencies’ and its sub-nodes. Without leading spaces, the value is not considered as a child node of ‘elm’.`, dependenciesPosition, "litvis:frontmatter:elm:dependencies");
            }
        }
    }
};
exports.lintElm = lintElm;
//# sourceMappingURL=lintElm.js.map